import { createContext, useContext, useEffect, useState } from "react";
import { initChatService, startChatService } from "@/feature/messenger/services/chat/ChatService";
import { useSession } from "next-auth/react";
import { ListConversationModel } from "@/feature/messenger/models/ListConversationModel";
import { ListMessagesModel } from "@/feature/messenger/models/ListMessagesModel";

const ChatConfigContext = createContext({});

const ChatConfigContextProvider = ({ children }: { children: React.ReactNode }) => {
  //state
  const [chatConfig, setChatConfig] = useState({})
  const [agentPermissions, setAgentPermissions] = useState({})
  const [chatbot, setChatbot] = useState({})
  const [id_chatbot, setIdChatbot] = useState("63")
  const [selectedSesion, setSelectedSesion] = useState<ListConversationModel>();
  const [listConversation, setListConversation] = useState<ListConversationModel[]>([]);
  const [listMessages, setListMessages] = useState<ListMessagesModel[]>([]);

  //search
  const [searchTerm, setSearchTerm] = useState('');

  //hooks
  const { data: session, status } = useSession()

  //functions
  const changeSelectedSesion = (sesion: ListConversationModel) => {
    setSelectedSesion(sesion);
  }

  const setListConversationHandler = async (data: ListConversationModel[]) => {
    setListConversation(data);
  }

  const addNewConversationHandler = async (data: ListConversationModel[]) => {
    setListConversationHandler([...listConversation, ...data]);
  }

  const deleteConversationBySesionHandler = async (data: ListConversationModel) => {
    const updatedList = listConversation.filter((item) => item.sesion !== data.sesion);
    setListConversationHandler(updatedList);
  }

  const updateListConversationHandler = async (data: ListConversationModel) => {
    const index = listConversation.findIndex((item) => item.sesion === data.sesion);
    if (index !== -1) {
      const updatedList = [...listConversation];
      updatedList[index] = data;
      setListConversationHandler(updatedList);
    }
  }

  useEffect(() => {
    if (status === "authenticated") {
      //const id_chatbot = session.user.chatbots[0].id_chatbot;
      startChatService(id_chatbot).then((res) => {
        setChatConfig(res.data);
      })
      initChatService(id_chatbot).then((res) => {
        setAgentPermissions(res.data.agentPermissions);
        setChatbot(res.data.chatbot);
        setListConversation(res.data.conversations);
      })
    }
  }, [])

  //messages 
  const setListMessagesHandler = async (data: ListMessagesModel[]) => {
    setListMessages(data);
  }

  const addNewMessagesHandler = async (data: ListMessagesModel[]) => {
    setListMessagesHandler([...listMessages, ...data]);
  }

  //search conversation
  const filteredConversations = listConversation.filter(item => {
    const nameComplete = `${item.nombre.toLowerCase()} ${item.sesion.toLowerCase()}`;
    return nameComplete.includes(searchTerm.toLowerCase());
  });

  const newlistConversation = searchTerm ? filteredConversations : listConversation;

  return <ChatConfigContext.Provider
    value={{
      chatConfig: chatConfig,
      agentPermissions: agentPermissions,
      chatbot: chatbot,
      id_chatbot: id_chatbot,
      selectedSesion: selectedSesion,
      listConversation: newlistConversation,
      changeSelectedSesion: changeSelectedSesion,
      setListConversationHandler: setListConversationHandler,
      setSearchTerm: setSearchTerm,
      listMessages: listMessages,
      setListMessagesHandler: setListMessagesHandler,
      addNewMessagesHandler: addNewMessagesHandler,
      addNewConversationHandler: addNewConversationHandler,
      updateListConversationHandler: updateListConversationHandler,
      deleteConversationBySesionHandler: deleteConversationBySesionHandler
    }}>
    {children}
  </ChatConfigContext.Provider>;
};
export default ChatConfigContextProvider;

export const useChatConfigContext = () => {
  return useContext(ChatConfigContext);
};